pub struct RequestInterceptConfiguration {
pub enabled: bool,
pub block_visuals: bool,
pub block_stylesheets: bool,
pub block_javascript: bool,
pub block_analytics: bool,
pub block_ads: bool,
pub intercept_manager: NetworkInterceptManager,
pub whitelist_patterns: Option<Vec<String>>,
pub blacklist_patterns: Option<Vec<String>>,
}Expand description
Chrome request interception configurations.
Fields
enabled: boolRequest interception enabled?
block_visuals: boolBlock visuals. By default this is enabled. This will prevent Prefetch, Ping, and some javascript from rendering.
block_stylesheets: boolBlock stylesheets.
block_javascript: boolBlock javascript only allowing critcal framework or lib based javascript to render..
block_analytics: boolBlock analytics.
block_ads: boolBlock ads. Requires the adblock feature flag.
intercept_manager: NetworkInterceptManagerIntercept Manager
whitelist_patterns: Option<Vec<String>>Whitelist patterns.
blacklist_patterns: Option<Vec<String>>Blacklist patterns.
Implementations
Sourceimpl RequestInterceptConfiguration
impl RequestInterceptConfiguration
Sourcepub fn new(enabled: bool) -> RequestInterceptConfiguration
pub fn new(enabled: bool) -> RequestInterceptConfiguration
Setup a new intercept config
Sourcepub fn new_manager(
enabled: bool,
url: &Option<Box<Url>>,
) -> RequestInterceptConfiguration
pub fn new_manager( enabled: bool, url: &Option<Box<Url>>, ) -> RequestInterceptConfiguration
Setup a new intercept config with a custom intercept manager.
Sourcepub fn setup_intercept_manager(&mut self, url: &Option<Box<Url>>)
pub fn setup_intercept_manager(&mut self, url: &Option<Box<Url>>)
Setup the network request manager type.
Sourcepub fn set_whitelist_patterns(
&mut self,
whitelist_patterns: Option<Vec<String>>,
)
pub fn set_whitelist_patterns( &mut self, whitelist_patterns: Option<Vec<String>>, )
Set the whitelist patterns.
Sourcepub fn set_blacklist_patterns(
&mut self,
blacklist_patterns: Option<Vec<String>>,
)
pub fn set_blacklist_patterns( &mut self, blacklist_patterns: Option<Vec<String>>, )
Set the blacklist patterns.