pub struct WaitForDelay {
pub timeout: Option<Duration>,
}Expand description
Wait for with a delay. Should only be used for testing purposes. This does nothing without the chrome flag enabled.
Fields
timeout: Option<Duration>The max time to wait. It is recommended to set this to a value around 30s. Set the value to None to remove the timeout.
Implementations