convert_abs_path

Function convert_abs_path 

pub fn convert_abs_path(base: &Url, href: &str) -> Url
Convert to absolute path. The base url must be the root path to avoid infinite appending. We always handle the urls from the base path.