clean_html_full

Function clean_html_full 

pub fn clean_html_full(html: &str) -> String
Clean the most extra properties in the html to fit the context. Removes nav/footer, trims meta, and prunes most attributes except id/class/data-*.