Application utils.
Structs
- The response of a web page.
Functions
- Perform a network request to a resource extracting all content as text streaming.
- Perform a network request to a resource extracting all content streaming.
- Log to console if configuration verbose.